#!perl -T

use strict;
use warnings;
use Test::More skip_all => 'Not yet implemented';

# XXX look at --ignore-dir, but apply to --ignore-file

# XXX is: match: ext: firstlinematch:
# XXX --no-ignorefile
